@@ -1,12 +1,7 @@
 import os
 import sys
 import wx
-import wx.stc
 import wx.adv
-import wx.xrc
-import wx.lib.splitter
-import wx.lib.ticker
-import wx.lib.iewin
 
 from threading import Thread
 
@@ -95,6 +90,12 @@ def threadJoin(key):
 def callAfter(handler,*args):
     wx.CallAfter(handler,*args)
    
+def doBusyJob(job,args=(),message="Please wait ...",parent=None,bgColor=None,fgColor=None):
+    import wx.lib.busy
+    with wx.lib.busy.BusyInfo(message,parent,bgColor,fgColor):
+        job(args)
+
+        
 ######################################################################
 # Layouts
 ######################################################################
@@ -226,16 +227,43 @@ class Simplebook(Book):
         super().__init__(layouts,parent,create,horizontal,expand,proportion,style='simple',key=key)
 
 class Panel(Control):
-    def __init__(self,layout,parent=None,create=False,expand=False,proportion=0,key=None):
+    import wx.lib.scrolledpanel
+    def __init__(self,layout,parent=None,create=False,expand=False,proportion=0,label="",style=None,key=None):
         super().__init__(key,expand,proportion)
         self.layout = layout
+        self.label = label
+        self.style = style
         if create is True and parent is not None:
             self.create(parent)
     def create(self,parent):
-        self.ctrl = wx.Panel( parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
-        self.sizer = makeLayout(self.layout,self.ctrl)
-        self.ctrl.SetSizer( self.sizer.ctrl )
-        self.ctrl.Layout()      
+        if self.style is None:
+            self.ctrl = wx.Panel( parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
+            self.sizer = makeLayout(self.layout,self.ctrl)
+            self.ctrl.SetSizer( self.sizer.ctrl )
+            self.ctrl.Layout()      
+        elif self.style == 'scroll':
+            self.ctrl = wx.lib.scrolledpanel.ScrolledPanel( parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.HSCROLL|wx.VSCROLL|wx.TAB_TRAVERSAL )
+            self.sizer = makeLayout(self.layout,self.ctrl)
+            self.ctrl.SetSizer( self.sizer.ctrl )
+            #begin for scrolledwindow
+            self.ctrl.SetAutoLayout(1)
+            #self.ctrl.SetupScrolling(scroll_y = True)
+            width = self.ctrl.GetBestSize().width
+            height = self.ctrl.GetBestSize().height
+            self.ctrl.SetSize((width, height))
+            self.ctrl.SetScrollbars( 1, 1, 1, 1 )              
+            self.sizer.ctrl.SetSizeHints(self.ctrl)
+            #end
+            self.ctrl.Layout()          
+        elif self.style == 'collapsible':
+            self.ctrl = wx.CollapsiblePane( parent, wx.ID_ANY, self.label, wx.DefaultPosition, wx.DefaultSize, wx.CP_DEFAULT_STYLE|wx.CP_NO_TLW_RESIZE )
+            pane = self.ctrl.GetPane()
+            self.sizer = makeLayout(self.layout,pane)
+            pane.SetSizer( self.sizer.ctrl )
+            self.ctrl.Expand()
+            self.sizer.ctrl.SetSizeHints(pane)
+            pane.Layout()      
+    
 
 class Scroll(Control): #TODO: Change ScrolledWindow -> ScrolledPane
     def __init__(self,layout,parent=None,create=False,horizontal=True,expand=False,proportion=0,key=None):
@@ -286,6 +314,7 @@ class Spliter1(Control):
 
     
 class Spliter(Control):
+    import wx.lib.splitter    
     def __init__(self,layouts,parent=None,create=False,style='vertical',expand=False,proportion=0,key=None):
         super().__init__(key,expand,proportion)
         self.layouts = layouts #left(top),right(bottom)
@@ -618,22 +647,25 @@ class List(Control):
         def ListCompareFunction(self, item1, item2):
     '''
     def __init__(self,choices=[],select=0,handler=None,expand=False,proportion=0,
-                 size=wx.DefaultSize,pos=wx.DefaultPosition,check=False,label="",multicol=False,edit=False,key=None):
+                 size=wx.DefaultSize,pos=wx.DefaultPosition,check=False,label="",style=None,key=None):
         super().__init__(key,expand,proportion,size,pos)
         self.choices = choices
         self.select = select
         self.handler = handler
         self.check = check
         self.label = label
-        self.multicol = multicol
-        self.edit = edit
+        self.style = style
     def create(self,parent):    
         import sys
         id = getId()
-        if self.edit is True:
-            self.ctrl = wx.adv.EditableListBox( parent, id, self.label, self.pos, self.size, 0 )
-            #TODO:
-        elif self.multicol is True:
+        if self.style is None:
+            if self.check is True:
+                self.ctrl = wx.CheckListBox( parent, id, self.pos, self.size, self.choices, 0 )
+            else:
+                self.ctrl = wx.ListBox( parent, id, self.pos, self.size, self.choices, 0 )
+            self.ctrl.SetSelection(self.select)
+            self.ctrl.Bind( wx.EVT_LISTBOX, self.handler, id=id )
+        elif self.style == 'multicol':
             self.ctrl = wx.ListCtrl(parent, id, style = wx.LC_REPORT)
             widths = []
             if len(self.choices) > 1:
@@ -647,16 +679,84 @@ class List(Control):
                     #self.ctrl.InsertColumn(col, label, align, width) 
                 for row in range(1,len(self.choices)):
                     self.ctrl.Append(self.choices[row]) 
-        else:
-            if self.check is True:
-                self.ctrl = wx.CheckListBox( parent, id, self.pos, self.size, self.choices, 0 )
-            else:
-                self.ctrl = wx.ListBox( parent, id, self.pos, self.size, self.choices, 0 )
-            self.ctrl.SetSelection(self.select)
-            self.ctrl.Bind( wx.EVT_LISTBOX, self.handler, id=id )
+        elif self.style == 'edit':
+            self.ctrl = wx.adv.EditableListBox( parent, id, self.label, self.pos, self.size, 0 )
+            #TODO:
         if self.key is not None:
             registerCtrl( self.key, self )
 
+class Picker(Control):
+    '''
+    Methods Summary
+    Properties Summary
+    '''
+    def __init__(self,style="",value=None,handler=None,expand=False,proportion=0,
+                 size=wx.DefaultSize,pos=wx.DefaultPosition,key=None):
+        super().__init__(key,expand,proportion,size,pos)
+        self.style = style
+        self.value = value
+        self.handler = handler
+    def create(self,parent):    
+        id = getId()
+        if self.style == 'dir': #value: path str
+            self.value = "" if self.value is None else self.value
+            self.ctrl = wx.DirPickerCtrl( parent, id, self.value, pos=self.pos, size=self.size, style=wx.DIRP_DEFAULT_STYLE )
+            self.ctrl.Bind( wx.EVT_DIRPICKER_CHANGED, self.handler, id=id )
+        elif self.style == 'file':
+            self.value = "" if self.value is None else self.value
+            self.ctrl = wx.FilePickerCtrl( parent, id, self.value, pos=self.pos, size=self.size, style=wx.FLP_DEFAULT_STYLE )
+            self.ctrl.Bind( wx.EVT_FILEPICKER_CHANGED, self.handler, id=id )
+        elif self.style == 'color': 
+            self.value = wx.BLACK if self.value is None else self.value
+            self.ctrl = wx.ColourPickerCtrl( parent, id, self.value, pos=self.pos, size=self.size, style=wx.CLRP_DEFAULT_STYLE )
+            self.ctrl.Bind( wx.EVT_COLOURPICKER_CHANGED, self.handler, id=id )
+        elif self.style == 'font':
+            self.value = wx.NullFont if self.value is None else self.value
+            self.ctrl = wx.FontPickerCtrl( parent, id, self.value, pos=self.pos, size=self.size, style=wx.FNTP_DEFAULT_STYLE )
+            self.ctrl.Bind( wx.EVT_FONTPICKER_CHANGED, self.handler, id=id )
+        elif self.style == 'date':
+            self.value = wx.DefaultDateTime if self.value is None else self.value
+            self.ctrl = wx.adv.DatePickerCtrl( parent, id, self.value, pos=self.pos, size=self.size, style=wx.adv.TP_DEFAULT )
+            self.ctrl.Bind( wx.adv.EVT_DATE_CHANGED, self.handler, id=id )
+        elif self.style == 'time':
+            self.value = wx.DefaultDateTime if self.value is None else self.value
+            self.ctrl = wx.adv.TimePickerCtrl( parent, id, self.value, pos=self.pos, size=self.size, style=wx.adv.TP_DEFAULT )
+            self.ctrl.Bind( wx.adv.EVT_TIME_CHANGED, self.handler, id=id )
+        else: #TODO: throw exception
+            pass 
+        if self.key is not None:
+            registerCtrl( self.key, self )
+            
+class DirPicker(Picker):
+    def __init__(self,style="",value=None,handler=None,expand=False,proportion=0,
+                 size=wx.DefaultSize,pos=wx.DefaultPosition,key=None):
+        super().__init__('dir',value,handler,expand,proportion,size,pos,key)
+
+class FilePicker(Picker):
+    def __init__(self,style="",value=None,handler=None,expand=False,proportion=0,
+                 size=wx.DefaultSize,pos=wx.DefaultPosition,key=None):
+        super().__init__('file',value,handler,expand,proportion,size,pos,key)
+               
+class ColorPicker(Picker):
+    def __init__(self,style="",value=None,handler=None,expand=False,proportion=0,
+                 size=wx.DefaultSize,pos=wx.DefaultPosition,key=None):
+        super().__init__('color',value,handler,expand,proportion,size,pos,key)
+        
+class FontPicker(Picker):
+    def __init__(self,style="",value=None,handler=None,expand=False,proportion=0,
+                 size=wx.DefaultSize,pos=wx.DefaultPosition,key=None):
+        super().__init__('font',value,handler,expand,proportion,size,pos,key)
+
+class DatePicker(Picker):
+    def __init__(self,style="",value=None,handler=None,expand=False,proportion=0,
+                 size=wx.DefaultSize,pos=wx.DefaultPosition,key=None):
+        super().__init__('date',value,handler,expand,proportion,size,pos,key)
+       
+class TimePicker(Picker):
+    def __init__(self,style="",value=None,handler=None,expand=False,proportion=0,
+                 size=wx.DefaultSize,pos=wx.DefaultPosition,key=None):
+        super().__init__('time',value,handler,expand,proportion,size,pos,key)
+                           
 class Progress(Control):
     '''
     Methods Summary
@@ -747,6 +847,7 @@ class StyledText(Control):
     Methods Summary
     Properties Summary
     '''
+    import wx.stc    
     def __init__(self,text="",expand=True,proportion=1,
                  size=wx.DefaultSize,pos=wx.DefaultPosition,key=None):
         super().__init__(key,expand,proportion,size,pos)
@@ -879,6 +980,7 @@ class Ticker(Control):
     Methods Summary
     Properties Summary
     '''
+    import wx.lib.ticker    
     def __init__(self,text="",fgcolor=wx.BLACK,bgcolor=wx.WHITE,expand=True,proportion=0,
                  size=wx.DefaultSize,pos=wx.DefaultPosition,key=None):
         super().__init__(key,expand,proportion,size,pos)
@@ -887,9 +989,13 @@ class Ticker(Control):
         self.bgcolor = bgcolor
     def create(self,parent):  
         self.ctrl = wx.lib.ticker.Ticker( parent, wx.ID_ANY, self.text, self.fgcolor, self.bgcolor, True, pos=self.pos, size=self.size, style=wx.NO_BORDER )
+        self.ctrl.Bind(wx.EVT_CLOSE, self.onClose)
         if self.key is not None:
             registerCtrl( self.key, self )
-            
+    def onClose(self,event):
+        self.ctrl.Stop()
+        event.Skip()
+    
 class Time(Control):
     '''
     Methods Summary
@@ -940,6 +1046,8 @@ class Tree(Control):
                 node = self.ctrl.AppendItem(parent,item)       
 
 class Web(Control):
+    import wx.lib.iewin
+    import wx.lib.pdfwin
     '''
     Methods Summary
         CanGoBack(self)
@@ -986,7 +1094,13 @@ class Web(Control):
                 self.ctrl.LoadUrl(self.url)
             if self.key is not None:
                 registerCtrl( self.key, self )
-        
+        if self.engine == 'pdf': #TODO: Do not work now : progID is not defined in PDFWindow.__init__()
+            self.ctrl = wx.lib.pdfwin.PDFWindow( parent, wx.ID_ANY, self.pos, self.size, 0 )
+            if self.url is not None:
+                self.ctrl.LoadFile(self.url)
+            if self.key is not None:
+                registerCtrl( self.key, self )
+
 ######################################################################
 # Dialogs
 ######################################################################
@@ -1050,6 +1164,9 @@ def onProgressDialog(dlg,percent):
 def progressDialogUpdate(dlg,percent):
     wx.CallAfter(onProgressDialog, dlg, percent)
 
+def CalendarDialog(parent=None,year=None,month=None,day=None):
+    dlg = wx.lib.CalenDlg(parent,month,day,year)
+    #TODO
 
 ######################################################################
 # WxApp
